      subroutine eddington_sp(ngridx, ngridy, deltay, umu, nlyr, 
     +        tskin, lyrtemp, lyrhgt, kexttot, salbtot, asymtot, 
     +        emis, ebar, lambert, tb)
C
C     CHRIS KUMMEROW
C     INCLUDES ASYMPTOTIC EXPRESSIONS FOR TERM3, TERM4, AND TERM5 IN
C     THE LIMIT OF SMALL EFFECTIVE OPTICAL DEPTHS; BILL OLSON FEB, 1995.
C
      include  '../GRIDparameters.inc'
     
      LOGICAL LAMBERT
      integer  ngridx, ngridy, nlyr
      real *4  lyrtemp(mxgridx,mxgridy,0:mxlyr) 
      real *4  lyrhgt(0:mxlyr)
      real *4  tskin(mxgridx,mxgridy)
      real *4  emis(mxgridx,mxgridy)
      real *4  ebar(mxgridx,mxgridy)
      real *4  kexttot(mxgridx,mxgridy,mxlyr)
      real *4  salbtot(mxgridx,mxgridy,mxlyr)
      real *4  asymtot(mxgridx,mxgridy,mxlyr)
      REAL *4  KEXT(MXLYR),
     $      SALB(MXLYR), ASYM(MXLYR), L(MXLYR), H(MXLYR),
     $      B0(MXLYR), B1(MXLYR), W(2*MXLYR,2*MXLYR),
     $      BB(2*MXLYR), DP(MXLYR), DM(MXLYR), Z(0:MXLYR)
      REAL *4  IOUT(0:MXLYR), I_IN(MXLYR+1,100), MU, NU
      REAL *4  TB(mxgridx,mxgridy)

      integer nymid

      real *4 deltay
      real *4 zmid
      integer *4 jin(0:mxlyr)
      integer *4 jout(0:mxlyr)

c
      data FISOT / 2.7 /    
c
c     Slant-Path Eddington computes Tb for one oblique radiance
c     path at a time.  The coordinate nx,ny is the gridpoint where
c     the radiance path intersects the earth's surface.

c     Calculate the mid-layer coordinates of the slant path for
c     both incoming and outgoing paths.

      do nz=0,nlyr 

        ! Find altitude of midlayer (or surface)
        if(nz .eq. 0) then
          zmid = lyrhgt(nz)
        else
          zmid = (lyrhgt(nz-1)+lyrhgt(nz))/2.
        end if

        ! Find nearest gridpoint to slant path intersection of midlayer.

        jin(nz) = nint(zmid*sqrt(1.-umu*umu)/(umu*deltay))
        jout(nz) = nint(-zmid*sqrt(1.-umu*umu)/(umu*deltay))
   
      end do


c     Loop over gridpoints.

      do nx = 1, ngridx
        do ny = 1, ngridy

    ! Construct 1-D "column" along incoming ray path

          Z(0) = LYRHGT(0)
          BTEMP = tskin(nx,ny)

          DO 20 J = 1,NLYR

    ! Index of path intersection of midlayer; grid is assumed
    ! periodic in y.
            nymid = mod(2*ngridy + ny + jin(j) - 1, ngridy) + 1
            
            Z(J)  = lyrhgt(J)
            B0(J) = lyrtemp(nx,nymid,J-1)
            B1(J) = (lyrtemp(nx,nymid,J) - 
     $              lyrtemp(nx,nymid,J-1))/
     $              (Z(J) - Z(J-1))
            KEXT(J) = kexttot(nx,nymid,j)
            SALB(J) = salbtot(nx,nymid,j)
            ASYM(J) = asymtot(nx,nymid,j)
            L(J) = SQRT(3.*KEXT(J)*KEXT(J)*(1. - SALB(J))*
     $            (1. - SALB(J)*ASYM(J)))
            H(J) = 1.5*KEXT(J)*(1. - SALB(J)*ASYM(J))

  20      CONTINUE
C
C         FILL IN THE MATRIX ELEMENTS WHICH FORM THE BOUNDARY CONDITIONS
C         AT THE TOP, BOTTOM AND LAYER INTERFACES OF THE CLOUD.  THERE ARE
C         TWO QUANTITIES, D+ "DP", AND D- "DM" AT EACH BOUNDARY, SO THE 
C         MATRIX HAS DIMENSIONS  2*NLYR BY 2*NLYR.
C         ORDER OF D'S:  D+(1),D-(1),D+(2),D-(2), .... , D+(NLYR),D-(NLYR)

C         SET ALL MATRIZ ELEMENTS TO ZERO	
          DO 45  I = 1,2*NLYR
            DO 45  J = 1,2*NLYR
              W(I,J) = 0.0
  45      CONTINUE	

C         FILL IN THE NON-ZERO MATRIX ELEMENTS
          W(1,1)   = ((EBAR(nx,ny) - 2.)*L(1)/H(1)) + EBAR(nx,ny)
          W(1,2)   = ((2. - EBAR(nx,ny))*L(1)/H(1)) + EBAR(nx,ny)
          DO 50  I = 2,2*(NLYR-1),2
            W(I,I-1)   =  (1. - L(I/2)/H(I/2))*EXP(+L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I,I  )   =  (1. + L(I/2)/H(I/2))*EXP(-L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I,I+1)   = -(1. - L(I/2+1)/H(I/2+1))
            W(I,I+2)   = -(1. + L(I/2+1)/H(I/2+1))

            W(I+1,I-1) =  (1. + L(I/2)/H(I/2))*EXP(+L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I+1,I)   =  (1. - L(I/2)/H(I/2))*EXP(-L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I+1,I+1) = -(1. + L(I/2+1)/H(I/2+1))
            W(I+1,I+2) = -(1. - L(I/2+1)/H(I/2+1))
  50      CONTINUE
          W(2*NLYR,2*NLYR-1) =  (1. + L(NLYR)/H(NLYR))*EXP(+L(NLYR)*
     $                          (Z(NLYR)-Z(NLYR-1)))
          W(2*NLYR,2*NLYR)   =  (1. - L(NLYR)/H(NLYR))*EXP(-L(NLYR)*
     $                          (Z(NLYR)-Z(NLYR-1)))

C         FILL IN THE ROW OF CONSTANTS IN THE LINEAR EQUATIONS
          BB(1)    = EBAR(nx,ny)*BTEMP - EBAR(nx,ny)*B0(1) - 
     $               (EBAR(nx,ny) - 2.)*B1(1)/H(1)
          DO 55  I = 2,2*(NLYR-1),2
            BB(I)   =  + B1(I/2)/H(I/2) - B1(I/2+1)/H(I/2+1)
            BB(I+1) =  - B1(I/2)/H(I/2) + B1(I/2+1)/H(I/2+1)
  55      CONTINUE
          BB(2*NLYR)  =  FISOT - B0(NLYR) - B1(NLYR)*(Z(NLYR) - 
     $                   Z(NLYR-1) + 1/H(NLYR))
C
C         MATRIX INVERSION IN DONE IN SUBROUTINE LINPAK
          CALL LINPAK(NLYR, W, BB, RCOND)
C
          DO 60  I = 1,NLYR
            DP(I) = BB(2*I-1)
            DM(I) = BB(2*I)
  60      CONTINUE

C         AFTER D'S ARE KNOWN, CALCULATE SURFACE RADIANCE

          MU = UMU
          NU = -UMU

C         FOR THE FOLLOWING CALCULATIONS, REFER TO APPENDIX B OF THESIS
C         *************************************************************
      
          IF ( LAMBERT ) THEN 
C           CALCULATE THE DOWNWELLING FLUX AT 81 ANGLES
            NANG = 81
            DO 997  NN = 1,NANG
              XNU = -(2.*NN - 1.)/(NANG*2.)
              I_IN(NLYR+1,NN) = FISOT
C             LOOP THROUGH THE REMAINING LAYERS
              DO 100  J = NLYR,1,-1
C               CALCULATE RADIANCE FROM TOP OF LAYER "J"
                XA = B0(J) - 1.5*SALB(J)*ASYM(J)*XNU*B1(J)/H(J)
                XB = B1(J)
                XC = SALB(J)*DP(J)*(1. - 1.5*ASYM(J)*XNU*L(J)/H(J))               
                XD = SALB(J)*DM(J)*(1. + 1.5*ASYM(J)*XNU*L(J)/H(J))
                YA = KEXT(J)/XNU
                YB = YA + L(J)
                YC = YA - L(J)
                DZ = Z(J) - Z(J-1)

                TERM1 = I_IN(J+1,NN)*EXP(YA*DZ)
                TERM2 = XA*(1. - EXP(YA*DZ))
                IF(ABS(ya*dz) .LT. 1.e-5) THEN
                 TERM3=-XB*YA*DZ*DZ
                ELSE
                 TERM3 = XB/YA*(EXP(YA*DZ)*(1. - YA*DZ) - 1.)
                ENDIF
                if(abs(yb*dz) .lt. 1.e-5) then
                 term4=-xc*ya*dz
                else
                 TERM4 = XC*YA/YB*(1. - EXP(YB*DZ))
                end if
                if(abs(yc*dz) .lt. 1.e-5) then
                 term5=-xd*ya*dz
                else
                 TERM5 = XD*YA/YC*(1. - EXP(YC*DZ))
                end if
                I_IN(J,NN) = TERM1 + TERM2 + TERM3 + TERM4 + TERM5
  100         CONTINUE
  997       CONTINUE
C
C           CALCULATE THE TOTAL DOWNWELLING FLUX REACHING THE SURFACE
            XIUP = 0.
            DO 47  NN = 1,NANG
              XIUP = XIUP + I_IN(1,NN)*(1./NANG)*(2.*NN-1.)/(2.*NANG)
  47        CONTINUE
            XIUP = 2.*XIUP
           
          ELSE

C           CALCULATE THE DOWNWELLING FLUX AT ANGLE MU ONLY
            NN = 22            ! THIS IS A DUMMY INDEX FOR I_IN
            I_IN(NLYR+1,NN) = FISOT
C           LOOP THROUGH THE REMAINING LAYERS
            DO 110  J = NLYR,1,-1
C             CALCULATE RADIANCE FROM TOP OF LAYER "J"
              XA = B0(J) - 1.5*SALB(J)*ASYM(J)*NU*B1(J)/H(J)
              XB = B1(J)
              XC = SALB(J)*DP(J)*(1. - 1.5*ASYM(J)*NU*L(J)/H(J))               
              XD = SALB(J)*DM(J)*(1. + 1.5*ASYM(J)*NU*L(J)/H(J))
              YA = KEXT(J)/NU
              YB = YA + L(J)
              YC = YA - L(J)
              DZ = Z(J) - Z(J-1)

              TERM1 = I_IN(J+1,NN)*EXP(YA*DZ)
              TERM2 = XA*(1. - EXP(YA*DZ))
              if(abs(ya*dz) .lt. 1.e-5) then
               term3=-xb*ya*dz*dz
              else
               TERM3 = XB/YA*(EXP(YA*DZ)*(1. - YA*DZ) - 1.)
              end if
              if(abs(yb*dz) .lt. 1.e-5) then
               term4=-xc*ya*dz
              else
               TERM4 = XC*YA/YB*(1. - EXP(YB*DZ))
              end if
              if(abs(yc*dz) .lt. 1.e-5) then
               term5=-xd*ya*dz
              else
               TERM5 = XD*YA/YC*(1. - EXP(YC*DZ))
              end if
              I_IN(J,NN) = TERM1 + TERM2 + TERM3 + TERM4 + TERM5
  110       CONTINUE
            XIUP = I_IN(1,22)
C
          ENDIF
C
          IOUT(0) = EMIS(nx,ny)*BTEMP + (1. - EMIS(nx,ny))*XIUP

c    Recalculate flux conditions along outgoing path.

    ! Construct 1-D "column" along outgoing ray path

          Z(0) = LYRHGT(0)
          BTEMP = tskin(nx,ny)

          DO 21  J = 1,NLYR

    ! Index of path intersection of midlayer; grid is assumed
    ! periodic in y.
            nymid = mod(2*ngridy + ny + jout(j) - 1, ngridy) + 1
            
            Z(J)  = lyrhgt(J)
            B0(J) = lyrtemp(nx,nymid,J-1)
            B1(J) = (lyrtemp(nx,nymid,J) - lyrtemp(nx,nymid,J-1))/
     $              (Z(J) - Z(J-1))
            KEXT(J) = kexttot(nx,nymid,j)
            SALB(J) = salbtot(nx,nymid,j)
            ASYM(J) = asymtot(nx,nymid,j)
            L(J) = SQRT(3.*KEXT(J)*KEXT(J)*(1. - SALB(J))*
     $            (1. - SALB(J)*ASYM(J)))
            H(J) = 1.5*KEXT(J)*(1. - SALB(J)*ASYM(J))

  21      CONTINUE
C
C         FILL IN THE MATRIX ELEMENTS WHICH FORM THE BOUNDARY CONDITIONS
C         AT THE TOP, BOTTOM AND LAYER INTERFACES OF THE CLOUD.  THERE ARE
C         TWO QUANTITIES, D+ "DP", AND D- "DM" AT EACH BOUNDARY, SO THE 
C         MATRIX HAS DIMENSIONS  2*NLYR BY 2*NLYR.
C         ORDER OF D'S:  D+(1),D-(1),D+(2),D-(2), .... , D+(NLYR),D-(NLYR)

C         SET ALL MATRIZ ELEMENTS TO ZERO	
          DO 46  I = 1,2*NLYR
            DO 46  J = 1,2*NLYR
              W(I,J) = 0.0
  46      CONTINUE	

C         FILL IN THE NON-ZERO MATRIX ELEMENTS
          W(1,1)   = ((EBAR(nx,ny) - 2.)*L(1)/H(1)) + EBAR(nx,ny)
          W(1,2)   = ((2. - EBAR(nx,ny))*L(1)/H(1)) + EBAR(nx,ny)
          DO 51  I = 2,2*(NLYR-1),2
            W(I,I-1)   =  (1. - L(I/2)/H(I/2))*EXP(+L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I,I  )   =  (1. + L(I/2)/H(I/2))*EXP(-L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I,I+1)   = -(1. - L(I/2+1)/H(I/2+1))
            W(I,I+2)   = -(1. + L(I/2+1)/H(I/2+1))

            W(I+1,I-1) =  (1. + L(I/2)/H(I/2))*EXP(+L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I+1,I)   =  (1. - L(I/2)/H(I/2))*EXP(-L(I/2)*(Z(I/2)-
     $                    Z(I/2-1)))
            W(I+1,I+1) = -(1. + L(I/2+1)/H(I/2+1))
            W(I+1,I+2) = -(1. - L(I/2+1)/H(I/2+1))
  51      CONTINUE
          W(2*NLYR,2*NLYR-1) =  (1. + L(NLYR)/H(NLYR))*EXP(+L(NLYR)*
     $                          (Z(NLYR)-Z(NLYR-1)))
          W(2*NLYR,2*NLYR)   =  (1. - L(NLYR)/H(NLYR))*EXP(-L(NLYR)*
     $                          (Z(NLYR)-Z(NLYR-1)))

C         FILL IN THE ROW OF CONSTANTS IN THE LINEAR EQUATIONS
          BB(1)    = EBAR(nx,ny)*BTEMP - EBAR(nx,ny)*B0(1) - 
     $               (EBAR(nx,ny) - 2.)*B1(1)/H(1)
          DO 56  I = 2,2*(NLYR-1),2
            BB(I)   =  + B1(I/2)/H(I/2) - B1(I/2+1)/H(I/2+1)
            BB(I+1) =  - B1(I/2)/H(I/2) + B1(I/2+1)/H(I/2+1)
  56      CONTINUE
          BB(2*NLYR)  =  FISOT - B0(NLYR) - B1(NLYR)*(Z(NLYR) - 
     $                   Z(NLYR-1) + 1/H(NLYR))
C
C         MATRIX INVERSION IN DONE IN SUBROUTINE LINPAK
          CALL LINPAK(NLYR, W, BB, RCOND)
C
          DO 61  I = 1,NLYR
            DP(I) = BB(2*I-1)
            DM(I) = BB(2*I)
  61      CONTINUE

          DO 101  J = 1,NLYR
C           CALCULATE THE UPWELLING RADIANCES AT THE TOP OF EACH LAYER J
            XA = B0(J) - 1.5*SALB(J)*ASYM(J)*MU*B1(J)/H(J)
            XB = B1(J)
            XC = SALB(J)*DP(J)*(1. - 1.5*ASYM(J)*MU*L(J)/H(J))               
            XD = SALB(J)*DM(J)*(1. + 1.5*ASYM(J)*MU*L(J)/H(J))
            YA = KEXT(J)/MU
            YB = YA + L(J)
            YC = YA - L(J)
            DZ = Z(J) - Z(J-1)
            TERM1 = IOUT(J-1)*EXP(-YA*DZ)
            TERM2 = XA*(1. - EXP(-YA*DZ))
            if(abs(ya*dz) .lt. 1.e-5) then
             term3=0.
            else
             TERM3 = XB/YA*(YA*DZ - 1. + EXP(-YA*DZ))
            end if
            if(abs(yb*dz) .lt. 1.e-5) then
             term4=xc*ya*dz*exp(-ya*dz)
            else
             TERM4 = XC*YA/YB*(EXP( (YB-YA)*DZ ) - EXP(-YA*DZ) )
            end if
            if(abs(yc*dz) .lt. 1.e-5) then
             term5=xd*ya*dz*exp(-ya*dz)
            else
             TERM5 = XD*YA/YC*EXP(-YA*DZ)*(EXP(YC*DZ) - 1.)
            end if
            IOUT(J) = TERM1 + TERM2 + TERM3 + TERM4 + TERM5
  101     CONTINUE
C
          TB(nx,ny) = IOUT(NLYR)

        end do
      end do

      return
      end
