C     The followingis a compilation of functions written by Peter
C     Bauer and Bill Olson.  
**
      FUNCTION ERF(X)
      IF(X.LT.0.)THEN
        ERF=-GAMMP(.5,X**2)
      ELSE
        ERF=GAMMP(.5,X**2)
      ENDIF
      RETURN
      END
**
      FUNCTION GAMMLN(XX)
      REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER
      DATA COF,STP/76.18009173D0,-86.50532033D0,24.01409822D0,
     *    -1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
      DATA HALF,ONE,FPF/0.5D0,1.0D0,5.5D0/
      X=XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X=X+ONE
        SER=SER+COF(J)/X
11    CONTINUE
      GAMMLN=TMP+LOG(STP*SER)
      RETURN
      END
**
      FUNCTION GAMMP(A,X)
      IF(X.LT.0..OR.A.LE.0.)PAUSE
      IF(X.LT.A+1.)THEN
        CALL GSER(GAMSER,A,X,GLN)
        GAMMP=GAMSER
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)
        GAMMP=1.-GAMMCF
      ENDIF
      RETURN
      END
**
      FUNCTION GAMMQ(A,X)
      IF(X.LT.0..OR.A.LE.0.)PAUSE
      IF(X.LT.A+1.)THEN
        CALL GSER(GAMSER,A,X,GLN)
        GAMMQ=1.-GAMSER
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)
        GAMMQ=GAMMCF
      ENDIF
      RETURN
      END
**
      SUBROUTINE GCF(GAMMCF,A,X,GLN)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      GLN=GAMMLN(A)
      GOLD=0.
      A0=1.
      A1=X
      B0=0.
      B1=1.
      FAC=1.
      DO 11 N=1,ITMAX
        AN=FLOAT(N)
        ANA=AN-A
        A0=(A1+A0*ANA)*FAC
        B0=(B1+B0*ANA)*FAC
        ANF=AN*FAC
        A1=X*A0+ANF*A1
        B1=X*B0+ANF*B1
        IF(A1.NE.0.)THEN
          FAC=1./A1
          G=B1*FAC
          IF(ABS((G-GOLD)/G).LT.EPS)GO TO 1
          GOLD=G
        ENDIF
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMMCF=EXP(-X+A*ALOG(X)-GLN)*G
      RETURN
      END
**      
      SUBROUTINE GSER(GAMSER,A,X,GLN)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      GLN=GAMMLN(A)
      IF(X.LE.0.)THEN
        IF(X.LT.0.)PAUSE
        GAMSER=0.
        RETURN
      ENDIF
      AP=A
      SUM=1./A
      DEL=SUM
      DO 11 N=1,ITMAX
        AP=AP+1.
        DEL=DEL*X/AP
        SUM=SUM+DEL
        IF(ABS(DEL).LT.ABS(SUM)*EPS)GO TO 1
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMSER=SUM*EXP(-X+A*LOG(X)-GLN)
      RETURN
      END
**
      REAL FUNCTION GAMMA(XX) 
C     This function computes the complete GAMMA function, 
C     as defined in "NUMERICAL RECIPES", Pag. 156 
      REAL*8 COF(6), STP, HALF, ONE, FPF, X, TMP, SER  
      DATA COF,STP/76.18009173D0, -86.50532033D0, 24.01409822D0, 
     > -1.231739516D0, .120858003D-2, -.536382D-5, 2.50662827465D0/ 
      DATA HALF, ONE, FPF/0.5D0,1.0D0,5.5D0/ 
      X=XX-ONE 
      TMP=X+FPF 
      TMP=(X+HALF)*LOG(TMP)-TMP 
      SER=ONE 
      DO J=1, 6 
        X=X+ONE 
        SER=SER+COF(J)/X 
      ENDDO 
      GAMMA=EXP(TMP+LOG(STP*SER)) 
      RETURN 
      END 
**
      function psi(t,p)
C     Return the water vapor diffusivity [m**2/s], given the 
C     temperature [K] and pressure p [mb].  Bill Olson  March, 1999
      real *4 t
      real *4 p
      real *4 psi
      psi=(8.794142e-4/(p*1000.))*(t**1.81)
      return
      end
**
      function mu(t)
C     Return the dynamic viscosity [kg/m-s] given the temperature [K]
C     Bill Olson  March, 1999
      implicit none
      real *4 t
      real *4 mu
      mu=(1.718e-5)*((8.7126161e-2)*(t**1.5)/(t+120.))
      return
      end
**
      function esatice(t)
C     DETERMINE THE SATURATION VAPOR PRESSURE WITH RESPECT 
C     TO ICE AT A GIVEN TEMPERATURE IN K FROM PRUPPACHER AND 
C     KLETT RANGE IS -50 C TO 0 C
      real *4 t
      real *4 tc
      real *4 esatice
      data a0/6.109177956e0/
      data a1/5.034698970e-1/
      data a2/1.886013408e-2/
      data a3/4.176223716e-4/
      data a4/5.824720280e-6/
      data a5/4.838803174e-8/
      data a6/1.838826904e-10/
      tc=t-273.15
      esatice=a0+tc*(a1+tc*(a2+tc*(a3+tc*(a4+tc*(a5+a6*tc)))))
      if(esatice .lt. 0.) esatice=0.
      return 
      end
**
      function esatvap(t)
C     DETERMINE THE SATURATION VAPOR PRESSURE WITH RESPECT 
C     TO LIQUID WATER AT A GIVEN TEMPERATURE IN K FROM 
C     PRUPPACHER AND KLETT RANGE IS -50 C TO +50 C
      real *4 t
      real *4 tc
      real *4 esatvap
      data a0/6.107799961e0/
      data a1/4.436518521e-1/
      data a2/1.428945805e-2/
      data a3/2.650648471e-4/
      data a4/3.031240396e-6/
      data a5/2.034080948e-8/
      data a6/6.136820929e-11/
      tc=t-273.15
      esatvap=a0+tc*(a1+tc*(a2+tc*(a3+tc*(a4+tc*(a5+a6*tc)))))
      if(esatvap .lt. 0.) esatvap=0.
      return 
      end
**      
      function iflr(x)
C     Return greatest integer in x
      integer *4 iflr
      real *4 x
      iflr=int(x)
      if(float(iflr) .gt. x) iflr=iflr-1
      return
      end
**      
      function ka(t)
C     Return the thermal conductivity [j/m-s-K] of air at 
C     temperature t [K].  Bill Olson  March, 1999
      implicit none
      real *4 t
      real *4 ka
      ka=(1.4144354e+2)*(1.496286e-5*(t**1.5)/(t+120.))
      return
      end
