      SUBROUTINE HOMOG3(M,T1,T2,OMG,N1,AM1,WMM,N2,AM2
     &    ,PR,PT,PR0,PT0,FSOL,NPLK1,CPLK,R,T,ER,ET,ZEIG
     &    ,Q,QI,C11,C22,VP,VM,DP,DM,ERR)
C$NAME  HOMOG3
C$LINK  AXB  CPCON  CSFN  EXPFN  GETQM GETQM2 TNVSS2
C SOLVE THE TRANSFER IN A HOMOGENEOUS SCATTERING AND EMITTING
C  MEDIUM BY THE DISCRETE ORDINATE METHOD.
C--- HISTORY
C 89.10.31 CREATED FROM HOMOG1 INCLUDING THERMAL RADIATION.
C 95.11.22 EPS=CCP(1)*10 -> EPS=EXP(LOG(CCP(1))*0.8)
C--- INPUT
C T1         R       OPTICAL DEPTH AT THE LAYER TOP.
C T2         R       OPTICAL DEPTH AT THE LAYER BOTTOM.
C OMG        R       SINGLE SCATTERING ALBEDO.
C N1         I       NO. OF THE QUADRATURE STREAMS.
C AM1     R(KNDM)    MU (I), I=1,N1.
C WMM     R(KNDM)    SQRT(W1/M1)
C N2         I       NO. OF THE SOLAR DIRECTIONS.
C AM2     R(KNA0)    MU0(I), I=1,N2.
C PR      R(KNDM,    SCALED P+-(I,J) I,J=1,N1
C           KNDM)
C PT                 SCALED P++(I,J)
C PR0     R(KNDM,    SCALED P0+-(I,J)  I=1,N1; J=1,N2.
C PT0       KNDM)    SCALED P0++(I,J)  I=1,N1; J=1,N2.
C FSOL       R       SOLAR IRRADIANCE AT THE TOP OF THE SYSTEM.
C NPLK1      I       MAX ORDER OF PLANK FUNCTION EXPANSION BY TAU + 1.
C                      IF NPLK1=0 THEN NO THERMAL.
C CPLK    R(NPLK1)   2*PI*(1-W)*B(N)
C--- OUTPUT
C R       R(KNDM,    REFLECTION   MATRIX   RIJ, I,J=1,N1.
C T         KNDM)    TRANSMISSION MATRIX   TIJ.
C ER      R(KNDM,    UPGOING   SOURCE MATRIX EU(I,J), I=1,N1; J=1,N2.
C ET        KNA0)    DOWNGOING SOURCE MATRIX ED(I,J).
C ZEIG    R(KNDM)    ROOT OF THE EIGENVALUES OF Z.
C Q                  Q-MATRIX
C QI                 INVERSE OF Q
C C11                C1 = INVERSE OF 2A-
C C22                C2 = INVERSE OF 2B-
C VP                 VS+
C VM                 VS-
C DP   R(KNDM,KPLK1) THERMAL EMISSION INTENSITY EXAPNSION (DNWARD)
C DM   R(KNDM,KPLK1) THERMAL EMISSION INTENSITY EXAPNSION (UPWARD)
C ERR      C*64      ERROR INDEX
C--- PARAMETER
C KNDM       I       DECLARED SIZE FOR NDA
C KNA0       I       DECLARED SIZE FOR NA0
C KPLK1      I       DECLARED SIZE FOR NPLK1
C$PRPC
c
      Include 'VISIRparameters.inc'
c
C      PARAMETER (KNDM=10,KNA0=10,KPLK1=2)
C$PSET GETQM.KNDM=KNDM; GETQM2.KNDM=KNDM
      PARAMETER (PI=3.141592653)
C--- AREAS FOR THE ROUTINE.
      CHARACTER ERR*(*)
      DIMENSION AM1(KNDM),WMM(KNDM),AM2(KNA0)
     &,PR(KNDM,KNDM),PT(KNDM,KNDM),PR0(KNDM,KNA0),PT0(KNDM,KNA0)
     &,CPLK(KPLK1),ZEIG(KNDM),R(KNDM,KNDM),T(KNDM,KNDM)
     &,ER(KNDM,KNA0),ET(KNDM,KNA0),Q(KNDM,KNDM),QI(KNDM,KNDM)
     &,C11(KNDM,KNDM),C22(KNDM,KNDM),VP(KNDM,KNA0),VM(KNDM,KNA0)
     &,DP(KNDM,KPLK1),DM(KNDM,KPLK1),DB(KNDM),QT(KNDM,KNDM)
     &,DPP(KNDM,KPLK1)
C--- WORKING AREAS
      PARAMETER (KROWIJ=KNDM*(KNDM+1)/2)
      DIMENSION CCP(3),X(KNDM,KNDM),Y(KNDM,KNDM),XI(KNDM,KNDM)
     &,SP(KNDM,KNA0),SM(KNDM,KNA0),G1(KNDM,KNA0),GAM(KNDM,KNA0)
     &,E0(KNA0),C(KNDM),SL(KNDM),SL1(KNDM)
     &,AP(KNDM,KNDM),AM(KNDM,KNDM),BP(KNDM,KNDM),BM(KNDM,KNDM)
     &,IW(KNDM2),DP0(KNDM),DP1(KNDM),DM0(KNDM),DM1(KNDM)
C PRECISION
      CALL CPCON(CCP)
C      EPS=CCP(1)*10
      EPS=EXP(LOG(CCP(1))*0.8)
C
      ERR=' '
      TAU=T2-T1
C X, Y MATRICES
      DO 1 I=1,N1
        DO 2 J=1,N1
        X(I,J)=-OMG*(PT(I,J)-PR(I,J))
    2   Y(I,J)=-OMG*(PT(I,J)+PR(I,J))
        X(I,I)=1.0/AM1(I)+X(I,I)
    1   Y(I,I)=1.0/AM1(I)+Y(I,I)
C DECOMPOSITION OF XY
      IF(M.EQ.0 .AND. 1.0-OMG.LE.EPS) THEN
        IW0=1
       ELSE
        IW0=0
      ENDIF
      IF(M.EQ.0 .AND. IW0.EQ.0 .AND. NPLK1.GT.0) THEN
        IPK=1
       ELSE
        IPK=0
      ENDIF
C ### GETQM2 ADDED BY MASUNAGA 00/07/05 ################
      IF (OMG.GE.1.E-5) THEN
         CALL GETQM(IW0,N1,X,Y,ZEIG,Q,QI,XI,IMN,ERR)
      ELSE
         CALL GETQM2(N1,X,Y,AM1,ZEIG,Q,QI,XI,ERR)
      ENDIF
C ######################################################
      IF(ERR.NE.' ') RETURN
C THERMAL SOURCE
CC C
      DO 35 I=1,N1
 35   DB(I)=0.
      IF(IPK.EQ.0) THEN
        IF(NPLK1.GT.0) THEN
          DO 25 J=1,NPLK1
          DO 25 I=1,N1
          DP(I,J)=0
   25     DM(I,J)=0
        ENDIF
       ELSE
        DO 3 I=1,N1
          SUM1=0
          DO 4 J=1,N1
C #### MODIFIED BY H.MASUNAGA 00/07/14 ############
    4     QT(I,J) = Q(J,I)
c ##################################################
          DO 5 J=NPLK1,1,-1
          IF(J+2.GT.NPLK1) THEN
            CPLK1=0
            ELSE
            CPLK1=DPP(I,J+2)
          ENDIF
C #### MODIFIED BY H.MASUNAGA 00/07/14 ############
    5     DPP(I,J) =((J+1)*J*CPLK1+WMM(I)*CPLK(J))/ZEIG(I)**2
c ##################################################
    3   CONTINUE
C #### ADDED BY H.MASUNAGA 00/07/14 ############
        CALL AXB(DP,QT,DPP,N1,N1,NPLK1,KNDM,KNDM,KNDM)
c ##################################################
CC D
        CALL AXB(DM,Q,DP,N1,N1,NPLK1,KNDM,KNDM,KNDM)
        DO 6 J=1,NPLK1
          DO 7 I=1,N1
          SUM=0
          IF(J+1.LE.NPLK1) THEN
            DO 8 K=1,N1
    8       SUM=SUM+QI(K,I)*DP(K,J+1)
          ENDIF
C ## DB(I) IS ZERO-TH-ORDER COEFFICIENT IN THE THERMAL SOURCE ###
C ## ADDED BY H.MASUNAGA (00/07/13) TO AVOID ROUND-OFF ERRORS.###
C ## DB(I) IS SUBTRACTED FROM DP(I,J) & DM(I,J) HERE AND ADDED###
C ## BACK WHEN ER(I,J) AND ET(I,J) ARE ESTIMATED LATER. THIS  ###
C ## PROCEDURE IS ESSENTIALLY IMPORTANT WHEN THE FIST-ORDER   ###
C ## COEFFICIENT IS EXTREMELY LARGER THAN THE ZERO-TH IN THE  ###
C ## THERMAL SOURCE, IN CASE ROUND-OFF ERRORS BECOME SERIOUS. ###
          IF (IPK.NE.0 .AND. J.EQ.1) DB(I)=DM(I,J)
          DP(I,J)=DM(I,J)-J*SUM
          DM(I,J)=DM(I,J)+J*SUM
          IF (J.EQ.1) THEN
             DP(I,J)=DP(I,J)-DB(I)
             DM(I,J)=DM(I,J)-DB(I)
          ENDIF
 7        CONTINUE
    6   CONTINUE
      ENDIF
C SIGMA+ - (FOR SINGLE SCATTERING)
      DO 9 I=1,N1
      DO 9 J=1,N2
      SP(I,J)=OMG*(PT0(I,J)+PR0(I,J))
    9 SM(I,J)=OMG*(PT0(I,J)-PR0(I,J))
C LOWER G
      DO 10 I=1,N1
      DO 10 J=1,N2
      SUM=0
      DO 11 K=1,N1
   11 SUM=SUM+X(I,K)*SP(K,J)
   10 G1(I,J)=-SUM-SM(I,J)/AM2(J)
C GAMMA
      CALL AXB(GAM,QI,G1,N1,N1,N2,KNDM,KNDM,KNDM)
      DO 12 I=1,N1
      DO 12 J=1,N2
   12 GAM(I,J)=GAM(I,J)/(1.0/AM2(J)**2-ZEIG(I)**2)
C VS+ AND -
      DO 14 J=1,N2
      TRNS0=-T1/AM2(J)
      TRNS0=EXPFN(TRNS0)*FSOL
      DO 14 I=1,N1
      SUM1=0.0
      SUM2=0.0
      DO 13 K=1,N1
      SUM1=SUM1+Q (I,K)*GAM(K,J)
   13 SUM2=SUM2+QI(K,I)*GAM(K,J)/AM2(J)+XI(I,K)*SM(K,J)
      VP(I,J)=(SUM1+SUM2)/2.0*TRNS0
   14 VM(I,J)=(SUM1-SUM2)/2.0*TRNS0
C E0
      DO 15 I=1,N2
      EX1=-TAU/AM2(I)
   15 E0(I)=EXPFN(EX1)
C BASE FUNCTION  C(TAU) AND S(TAU).
      DO 16 I=1,N1
   16 CALL CSFN(TAU,TAU,ZEIG(I),C(I),SL(I),SL1(I))
C A+-, B+-
      DO 17 I=1,N1
      DO 17 J=1,N1
      SUM1=Q (I,J)*C  (J)
      SUM2=QI(J,I)*SL (J)
      SUM3=Q (I,J)*SL1(J)
      SUM4=QI(J,I)*C  (J)
      AP(I,J)=SUM1-SUM2
      AM(I,J)=SUM1+SUM2
      BP(I,J)=SUM3-SUM4
   17 BM(I,J)=SUM3+SUM4
C C11 AND C22 -> THEIR INVERSION.
      DO 18 I=1,N1
      DO 18 J=1,N1
      C11(I,J)=2.0*AM(I,J)
   18 C22(I,J)=2.0*BM(I,J)
      CALL TNVSS2(N1,C11,DT,0.0,KNDM,IW,ERR)
      IF(ERR.NE.' ') THEN
        ERR='ERROR TO GET -C11- (HOMOG3)'
        RETURN
      ENDIF
      CALL TNVSS2(N1,C22,DT,0.0,KNDM,IW,ERR)
      IF(ERR.NE.' ') THEN
        ERR='ERROR TO GET -C22- (HOMOG3)'
        RETURN
      ENDIF
C R, T MATRICES
      DO 19 I=1,N1
      DO 19 J=1,N1
      SUM1=0
      SUM2=0
      DO 20 K=1,N1
      SUM1=SUM1+AP(I,K)*C11(K,J)
   20 SUM2=SUM2+BP(I,K)*C22(K,J)
      R(I,J)=SUM1+SUM2
   19 T(I,J)=SUM1-SUM2
C ER, ET MATRICES
      DO 22 I=1,N1
      DP1(I)=0
      DM1(I)=0
      IF(IPK.EQ.1) THEN
        DO 21 J=1,NPLK1
        DP1(I)=DP1(I)+DP(I,J)*TAU**(J-1)
   21   DM1(I)=DM1(I)+DM(I,J)*TAU**(J-1)
        DP0(I)=DP(I,1)
        DM0(I)=DM(I,1)
       ELSE
        DP0(I)=0
        DM0(I)=0
      ENDIF
   22 CONTINUE
      DO 23 J=1,N2
      DO 23 I=1,N1
      SUM1=0
      SUM2=0
      DO 24 K=1,N1
      VP0=VP(K,J)      +DP0(K)
      VM1=VM(K,J)*E0(J)+DM1(K)
      SUM1=SUM1+R(I,K)*VP0+T(I,K)*VM1
   24 SUM2=SUM2+T(I,K)*VP0+R(I,K)*VM1
      ER(I,J)=VM(I,J)      +DM0(I)-SUM1
   23 ET(I,J)=VP(I,J)*E0(J)+DP1(I)-SUM2
C      ER(I,J)=VM(I,J)      +MAX(DM0(I)-SUM1,0.)
C   23 ET(I,J)=VP(I,J)*E0(J)+MAX(DP1(I)-SUM2,0.)
C ## ADDED BY MASUNAGA (00/07/13) #############################
C ## THE ZERO-TH-ORDER COEFFICIENT, DB(I), SUBTRACTED ABOVE ###
C ## IS RESTORED HERE.                                      ###
      DO 30 J=1,N2
      DO 30 I=1,N1
      SUM=0
      DO 31 K=1,N1
 31   SUM=SUM+(T(I,K)+R(I,K))*DB(K)
      ER(I,J)=ER(I,J)+DB(I)-SUM
 30   ET(I,J)=ET(I,J)+DB(I)-SUM
      DO 32 I=1,N1
      DP(I,1)=DP(I,1)+DB(I)
      DM(I,1)=DM(I,1)+DB(I)
 32   CONTINUE
C #############################################################
      RETURN
      END
