      SUBROUTINE INTCR1(IMTHD,AM0,AM1U,CS1,NTAU,UTAU,UTAUT
     &,NLN,THK,THKT,OMG,OMGT,PHS,PHST,FF,KLGN1,MXLGN1,NLGN1
     &,NLGT1,G,EPSP,NCHK1,COR,SGL2,PHSB,ERR)
C SIGLE AND SECOND SCATTERING CORRECTION
C ASSUME DELTA-M METHOD
C--- HISTORY
C 90. 1.28 CREATED
C--- INPUT
C IMTHD      I         1: MS, 2: TMS, 3: IMS
C AM0        R         COS(SOLAR ZENITH ANGLE)
C AM1U       R         COS(EMERGENT NADIR ANGLE)
C                     .LT.O: UPWARD,  .GT.0: DOWNWARD
C CS1        R         COS(SCATTERING ANGLE)
C NTAU       I         NUMBER OF USER DEFINED ANGLE
C UTAU     R(NTAU)     USER DEFINED OPTICAL DEPTH
C UTAUT    R(NTAU)     TRUNCATED USER DEFINED DEPTH
C NLN        I         NUMBER OF LAYER
C THK      R(NLN)      OPTICAL THICKNESS OF SUBLAYER
C THKT     R(NLN)      TRUNCATED THICKNESS
C OMG      R(NLN)      SINGLE SCATTERING ALEBEDO
C OMGT     R(NLN)      TRUNCATED SINGLE SCATTERING ALBEDO
C PHS      R(NLN)      PHASE FUNCTION  (INTEGRAL = 1 OVER UNIT SPHERE)
C PHST     R(NLN)      TRUNCATED PHASE FUNCTION (1 OVER UNIT SPHERE)
C FF       R(NLN)      TRUNCATION FRACTION
C KLGN1      I         FIRST ARGUMENT SIZE OF G
C MXLGN1     I         MAX(NLGN1) (ONLY FOR IMTHD=3)
C NLGN1    I(NLN)      MAX ORDER OF LEGENDER SERIES +1 IN EACH SUBLAYER
C                        (ONLY FOR IMTHD=3)
C NLGT1    I(NLN)      SAME AS -NLGN1- BUT FOR TRUNCATION
C G      R(KLGN1,NLN)  PHASE FUNCTION MOMENT (ONLY FOR IMTHD=3)
C EPSP       R         CONVERGENCE CRITERION OF LEGENDRE SERIS OF
C                        PHASE FUNCTION ** 2
C NCHK1      I         NUMBER OF CONSECTIVE CONVERGENCES BEFORE
C                        FINAL DECISION
C--- OUTPUT
C COR      R(NTAU)     AI= AI+COR*FSOL IS CORRECTED INTENSITY
C                        AT EACH USER
C ERR      C*64        ERROR INDICATER
C                       DEFINED DEPTH (UTAU)
C--- WORK
C SGL2     R(NTAU)
C PHSB     R(NLN)
C--- AREA FOR THIS ROUTINE
      PARAMETER (PI=3.141592654, RAD=PI/180.0)
      CHARACTER ERR*(*)
      DIMENSION UTAU(*),UTAUT(*),THK(*),THKT(*)
     &,OMG(*),OMGT(*),PHS(*),PHST(*),FF(*),NLGN1(*)
     &,NLGT1(*),G(KLGN1,*),COR(*),SGL2(*),PHSB(*)
C
      ERR=' '
      DO 1 IT=1,NTAU
    1 COR(IT)=0
      IF(IMTHD.LE.0 .OR. IMTHD.GE.4) RETURN
C--- MS-METHOD IN REFERENCE-NT
      IF(IMTHD.EQ.1) THEN
C SGL2 = U1WAVE OF EQ.(14)
        DO 2 L=1,NLN
    2   PHSB(L)=(1-FF(L))*PHST(L)
        DO 3 IT=1,NTAU
    3   SGL2(IT)=SGLR(AM1U,AM0,NLN,THK,OMG,PHSB,UTAU(IT))
C CORRECTION BY MS-METHOD.  SEE EQ.(14)
        DO 4 IT=1,NTAU
CC SGL1 = U1 OF EQ.(14)
        SGL1=SGLR(AM1U,AM0,NLN,THK,OMG,PHS,UTAU(IT))
    4   COR(IT)=-SGL2(IT)+SGL1
        RETURN
      ENDIF
C--- TMS AND IMS METHODS
C  SGL2 = U1* OF EQ.(15)
      DO 5 IT=1,NTAU
    5 SGL2(IT)=SGLR(AM1U,AM0,NLN,THKT,OMGT,PHST,UTAUT(IT))
      DO 6 L=1,NLN
    6 PHSB(L)=PHS(L)/(1-FF(L))
C CORRECTION OF INTENSITY BY TMS-METHOD.  SEE EQ.(15)
      DO 7 IT=1,NTAU
CC  SGL1 = U1WAVE* OF EQ.(15)
      SGL1=SGLR(AM1U,AM0,NLN,THKT,OMGT,PHSB,UTAUT(IT))
    7 COR(IT)=-SGL2(IT)+SGL1
      IF(IMTHD.EQ.2 .OR. AM1U.LE.0.0) RETURN
C--- SECONDARY SCATTERING CORRECTION FOR IMS-METHOD
      DO 8 IT=1,NTAU
        UTAUS=UTAU(IT)
C GETTING MEAN OPTICAL CONSTANTS ABOVE THE USER DEFINED LEVEL-UTAUS
        EH=0
        SH=0
        SHH=0
        PHSPK=0
        DPT2=0
        DO 9 L=1,NLN
          DPT1=DPT2
          DPT2=DPT1+THK(L)
          IF(UTAUS.LE.DPT1) GOTO 10
          IF(UTAUS.LT.DPT2) THEN
            TAU=UTAUS-DPT1
           ELSE
            TAU=THK(L)
          ENDIF
          EH= EH     +       TAU
          SH= SH     +OMG(L)*TAU
          SHH=SHH    +OMG(L)*TAU*FF(L)
    9     PHSPK=PHSPK+OMG(L)*TAU*(PHS(L)-(1-FF(L))*PHST(L))
   10   IF(ABS(EH).LE.0) GOTO 8
C WH: MEAN SINGLE SCATTERING ALBEDO
        WH=SH/EH
C FH: MEAN TRUNCATED FRACTION
        FH=SHH/SH
        IF(FH.LE.0.0) GOTO 8
C PHSPK: MEAN TRUNCETED PEAK OF PHASE FUNCTION
        PHSPK=PHSPK/SHH
C AM3: VARIABLE APPEARING IN EQ.(23)
        AM3=AM0/(1-FH*WH)
C LEGENDRE SUM FOR EQ.(23)
C  WE TRUNCATE THE SERIES WHEN ABS(GPK**2) BECOMES SMALLER THAN -EPSP-
C   FOR SUCCESSIVE THREE TIMES.
        PHSPK2=0
        INIT=1
        ICHK=0
        DO 11 K1=1,MXLGN1
CC PHSPK: 2PHAT-PHAT2 OF EQ.(23)
CC  MEAN VALUE FOR THE LAYER ABOVE THE USER-DEFINED LEVEL
          GPK=0
          DPT2=0
          DO 12 L=1,NLN
          DPT1=DPT2
          DPT2=DPT1+THK(L)
          IF(K1.LE.NLGN1(L) .AND. OMG(L).GT.0) THEN
            IF(UTAUS.LE.DPT1) GOTO 13
            IF(UTAUS.LT.DPT2) THEN
              TAU=UTAUS-DPT1
             ELSE
              TAU=THK(L)
            ENDIF
            IF(K1.LE.NLGT1(L)) THEN
              GP=FF(L)
             ELSE
              GP=G(K1,L)
            ENDIF
            GPK=GPK+GP*OMG(L)*TAU
          ENDIF
   12     CONTINUE
   13     GPK=GPK/SHH
          GPK2=GPK**2
          IF(GPK2.LE.EPSP) THEN
            ICHK=ICHK+1
            IF(ICHK.GE.NCHK1) GOTO 14
          ENDIF
   11     PHSPK2=PHSPK2+(2*K1-1)*GPK2*PLGD(INIT,CS1)
   14   PHSPK2=PHSPK2/4/PI
CC CORRECTION BY UU = UU + UHAT IN IMS-METHOD
CC   SGL3: UHAT OF EQ.(23)
        SGL3=(FH*WH)**2/(1-FH*WH)*(2*PHSPK-PHSPK2)
     &    * HF(UTAUS, AM1U, AM3, AM3)
        COR(IT)=COR(IT)-SGL3
    8 CONTINUE
      RETURN
      END
