      SUBROUTINE OCNR11(M,NDA,AMUA,WA,CR,CI,U10,R)
C REFLECTION MATRIX OF OCEAN SURFACE
C--- HISTORY
C 92. 9. 1 CREATED BY HASUMI
C    12.23 MODIFIED BY NAKAJIMA
C 95. 6. 2 Generated from OCNRF1
C--- INPUT
C M      I     FOURIER ORDER
C KNDM   I     DECLARED DIMENSION OF R AND AMUA
C NDA    I     USED DIMENSION OF R AND AMUA
C AMUA  R(NDA) QUADRATURE POINTS IN HEMISPHERE
C              DECREASING ORDER (ZENITH TO HORIZON, OR, 1 -> 0)
C WA    R(NDA) QUADRATURE WEIGHTS
C EM     R     RELATIVE REFRACTIVE INDEX
C              ABOUT 1.33 FROM ATMOSPHERE TO OCEAN
C              ABOUT 1/1.33 FROM OCEAN TO ATMOSPHERE
C U10    R     WIND VELOCITY AT 10 M ABOVE THE SURFACE
C CR       R    Relative refractive index of the media
C               About 1.33 for atmosphere to ocean incidence,
C               and 1/1.33 for ocean to atmosphere incidence.
C CI       R    Relative refractive index for imaginary part
C               M = CR + I*CI
C--- OUTPUT
C R    R(KNDA,NDA)  REFLECTION MATRIX FOR M-TH FOURIER ORDER
C                    ur = R * ui
C$END
C
      Include 'VISIRparameters.inc'
c
      SAVE
C      PARAMETER (KNDM  =10)
      PARAMETER(PI=3.141592654)
      DIMENSION AMUA(KNDM),WA(KNDM),R(KNDM, KNDM)
C LOCAL VARIABLES
      PARAMETER (KN=30,KNDM1=KNDM+1)
      DIMENSION X(KNDM1),GX(KN),GW(KN),FI1(KN),FI2(KN),COSM1(KN)
     & ,COSM2(KN),XB(5,2),NS(2),RR(KNDM, KNDM)
      DATA INIT/1/
C
      IF(INIT.GT.0) THEN
        N=KN
        INIT=0
        CALL QGAUSN(GW, GX, N )
      ENDIF
C Parameters for integration
      SIGMA=SQRT(0.00534*U10)
      DISPA=ATAN(SIGMA)
      COSDS=COS(DISPA)
      DO 4 K=1,N
      DFI2=PI-DISPA
      FI1(K)=DISPA+DFI2*GX(K)
      FI2(K)=DISPA*GX(K)
      COSM1(K)=2*COS(FI1(K)*M)*DFI2 *GW(K)
    4 COSM2(K)=2*COS(FI2(K)*M)*DISPA*GW(K)
      X(1)=1
      IF(NDA.GE.2) THEN
        DO 1 I = 2, NDA
    1   X(I)=(AMUA(I-1)+AMUA(I))/2
      ENDIF
      X(NDA+1)=0
C
      DO 2 I = 1, NDA
CC X1=COS(THETA(I)-DISPA), X2=COS(THETA(I)+DISPA)
CC   WHERE AMUA(I)=COS(THETA(I))
      B=SQRT((1-AMUA(I)**2)*(1-COSDS**2))
      X1=AMUA(I)*COSDS+B
      X2=AMUA(I)*COSDS-B
CC SETING MU-BAOUDARY FOR MU-INEGRATION
      NS1=1
      XB(1,1)=X(I)
      IF(X1.LT.X(I)) THEN
        NS1=NS1+1
        XB(NS1,1)=X1
      ENDIF
      NS1=NS1+1
      XB(NS1,1)=AMUA(I)
      IF(X2.GT.X(I+1)) THEN
        NS1=NS1+1
        XB(NS1,1)=X2
      ENDIF
      NS1=NS1+1
      XB(NS1,1)=X(I+1)
      NS(1)=NS1-1
      NS(2)=1
      XB(1,2)=X(I)
      XB(2,2)=X(I+1)
C
      DO 2 J = 1, NDA
      IF(I.EQ.J) THEN
        IEQ=1
       ELSE
        IEQ=2
      ENDIF
CC MU-INTEGRATION
      RIJ=0
      AMI = AMUA(J)
        DO 3 IS=1,NS(IEQ)
        DX=XB(IS,IEQ)-XB(IS+1,IEQ)
        DO 3 II = 1, N
          AME=XB(IS+1,IEQ)+DX*GX(II)
          W=DX*GW(II)*AME
          DO 3 K = 1, N
    3     RIJ=RIJ+(COSM1(K)*SEARF1(AME,AMI,FI1(K),U10,CR,CI)
     &            +COSM2(K)*SEARF1(AME,AMI,FI2(K),U10,CR,CI))*W
    2 RR(I,J)=RIJ/WA(I)
C SYMMETRIC OPERATION
      DO 5 I=1,NDA
      DO 5 J=1,I
      RRR=(RR(I,J)+RR(J,I))/2
      R(I,J)=RRR/AMUA(I)*WA(J)
    5 R(J,I)=RRR/AMUA(J)*WA(I)
      RETURN
      END
      SUBROUTINE OCNR31(M,NDA,AMUA,WA,AM0,CR,CI,U10,SR)
C REFLECTION MATRIX OF OCEAN SURFACE
C--- HISTORY
C 92. 9. 1 CREATED BY HASUMI
C    12.23 MODIFIED BY NAKAJIMA
C 93. 3.22 /WA(I) debugged by Takashi
C     3.29 AMI -> AM0
C--- INPUT
C M      I     FOURIER ORDER
C KNDM   I     DECLARED DIMENSION OF R AND AMUA
C NDA    I     USED DIMENSION OF R AND AMUA
C AMUA  R(NDA) QUADRATURE POINTS IN HEMISPHERE
C              DECREASING ORDER (ZENITH TO HORIZON, OR, 1 -> 0)
C WA    R(NDA) Quadrature weights
C AM0    R     Cos (Solar Zenith Angle)
C CR       R    Relative refractive index of the media
C               About 1.33 for atmosphere to ocean incidence,
C               and 1/1.33 for ocean to atmosphere incidence.
C CI       R    Relative refractive index for imaginary part
C               M = CR + I*CI
C U10    R     WIND VELOCITY AT 10 M ABOVE THE SURFACE
C--- OUTPUT
C SR    SR(KNDA)  REFLECTION SOURCE MATRIX FOR M-TH FOURIER ORDER
C$END
C
      Include 'VISIRparameters.inc'
c
      SAVE
C      PARAMETER (KNDM  =10)
      PARAMETER(PI=3.141592654)
      DIMENSION AMUA(KNDM),WA(KNDM),SR(KNDM)
C LOCAL VARIABLES
      PARAMETER (KN=30,KNDM1=KNDM+1)
      DIMENSION X(KNDM1),GX(KN),GW(KN),FI1(KN),FI2(KN),COSM1(KN)
     & ,COSM2(KN),XB(5,2),NS(2)
      DATA INIT/1/
C
      IF(INIT.GT.0) THEN
        N=KN
        INIT=0
        CALL QGAUSN(GW, GX, N )
      ENDIF
C Parameters for integration
      SIGMA=SQRT(0.00534*U10)
      DISPA=ATAN(SIGMA)
      COSDS=COS(DISPA)
      DO 4 K=1,N
      DFI2=PI-DISPA
      FI1(K)=DISPA+DFI2*GX(K)
      FI2(K)=DISPA*GX(K)
      COSM1(K)=2*COS(FI1(K)*M)*DFI2 *GW(K)
    4 COSM2(K)=2*COS(FI2(K)*M)*DISPA*GW(K)
      X(1)=1
      IF(NDA.GE.2) THEN
        DO 1 I = 2, NDA
    1   X(I)=(AMUA(I-1)+AMUA(I))/2
      ENDIF
      X(NDA+1)=0
C
      DO 2 I = 1, NDA
CC X1=COS(THETA(I)-DISPA), X2=COS(THETA(I)+DISPA)
CC   WHERE AMUA(I)=COS(THETA(I))
      B=SQRT((1-AMUA(I)**2)*(1-COSDS**2))
      X1=AMUA(I)*COSDS+B
      X2=AMUA(I)*COSDS-B
CC SETING MU-BAOUDARY FOR MU-INEGRATION
      NS1=1
      XB(1,1)=X(I)
      IF(X1.LT.X(I)) THEN
        NS1=NS1+1
        XB(NS1,1)=X1
      ENDIF
      NS1=NS1+1
      XB(NS1,1)=AMUA(I)
      IF(X2.GT.X(I+1)) THEN
        NS1=NS1+1
        XB(NS1,1)=X2
      ENDIF
      NS1=NS1+1
      XB(NS1,1)=X(I+1)
      NS(1)=NS1-1
      NS(2)=1
      XB(1,2)=X(I)
      XB(2,2)=X(I+1)
C
      IF(AMUA(I).EQ.AM0) THEN
        IEQ=1
       ELSE
        IEQ=2
      ENDIF
CC MU-INTEGRATION
      RIJ=0
        DO 3 IS=1,NS(IEQ)
        DX=XB(IS,IEQ)-XB(IS+1,IEQ)
        DO 3 II = 1, N
          AME=XB(IS+1,IEQ)+DX*GX(II)
          W=DX*GW(II)*AME
          DO 3 K = 1, N
    3     RIJ=RIJ+(COSM1(K)*SEARF1(AME,AM0,FI1(K),U10,CR,CI)
     &            +COSM2(K)*SEARF1(AME,AM0,FI2(K),U10,CR,CI))*W
    2 SR(I)=RIJ/AMUA(I)/WA(I)
      RETURN
      END
