      subroutine WVCAL(IND,P,T,RH,PPMV,GM3,E,GM3S,ES)
C$NAME WVCAL
C$LINK WVSAT
C Conversion of units for water vapor
C PPMV =N/Na =e/(P-e)*1.0E6, RH =E/ES =GM3/GM3S
C eV = NRT, (P-e)V = Na RT
C--- history
C 95. 9.30  Created
C--- input
C IND   I    1: Give RH  (relative humidity, 0-1)
C            2: Give PPMV (volume mixing ratio in ppmV)
C            3: Give GM3 (mass mixing ratio in g/m3)
C            4: Give E    (water vapor pressure in hPa)
C P     R    Atmospheric pressure (hPa)
C T     R    Temperature (K)
C One of RH, PPMV, GM3, or E
C--- output
C Other than input variable among RH, PPMV and GM3.
C GM3S  R    Saturation water vapor content (g/m3)
C ES    R    Saturation water vapor pressure (hPa)
C--- Gas constant (cgs) and molecular weights of water vapor
C           and air (g/mol)
      parameter (R=8.314e7, W=18.02, AIR=28.964)
C Satuaration water vapor (g/cm3)
      GCM3S=WVSAT(T)*1.0E-6
      ES=GCM3S/W*R*T/1.0E3
      if(IND.eq.1) then
        GCM3=RH*GCM3S
        E=RH*ES
        PPMV=E/(P-E)*1.0E6
       else if(IND.eq.2) then
        E=P*PPMV/(1.0E6+PPMV)
        RH=E/ES
        GCM3=RH*GCM3S
       else if(IND.eq.3) then
        GCM3=GM3*1.0E-6
        RH=GCM3/GCM3S
        E=RH*ES
        PPMV=E/(P-E)*1.0E6
       else if(IND.eq.4) then
        RH=E/ES
        GCM3=RH*GCM3S
        PPMV=E/(P-E)*1.0E6
      endif
      GM3=GCM3*1.0E6
      GM3S=GCM3S*1.0E6
      return
      end
      FUNCTION WVSAT(T)
C$NAME WVSAT
C Saturation water vapor amount (Lowtran)
C  Ws=A*exp(18.9766-14.9595A-2.4388A**2) g/m3
C  A=T0/T, T0=273.15, T in K (T0-50,T0+50)
C--- history
C 93. 3. 3 Created
C--- Input
C T       R      Absolute temperature (K)
C                T0-50 to T0+50
C--- Output
C WVSAT  RF      Water vapor amount (g/m3)
C---
      A=273.15/T
      WVSAT=A*exp(18.9766-14.9595*A-2.4388*A**2)
      RETURN
      END
